﻿// Copyright (c) 2020 Tudor Stanciu

using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using System;
using System.Threading.Tasks;
using Tuitio.Application.Services.Abstractions;
using Tuitio.Application.Stores;
using Tuitio.Domain.Entities;
using Tuitio.Domain.Repositories;

namespace Tuitio.Application.Services
{
    internal class BehaviorService : IBehaviorService
    {
        private readonly IServiceProvider _serviceProvider;
        private readonly ILogger<BehaviorService> _logger;
        private readonly ITokenService _tokenService;
        private readonly ITokenStore _securityStore;

        public BehaviorService(IServiceProvider serviceProvider, ILogger<BehaviorService> logger, ITokenService tokenService, ITokenStore securityStore)
        {
            _serviceProvider=serviceProvider;
            _logger=logger;
            _tokenService=tokenService;
            _securityStore=securityStore;
        }

        public void FillTokenStore()
            => FillTokenStoreAsync().GetAwaiter().GetResult();

        public async Task FillTokenStoreAsync()
        {
            var activeTokens = Array.Empty<UserToken>();
            using (var scope = _serviceProvider.CreateScope())
            {
                var _repository = scope.ServiceProvider.GetRequiredService<IUserRepository>();
                activeTokens = await _repository.GetActiveTokens();
            }

            if (activeTokens.Length <= 0)
                return;

            _logger.LogInformation($"BehaviorService: {activeTokens.Length} active tokens were found in database.");
            foreach (var token in activeTokens)
            {
                var storeToken = _tokenService.ExtractToken(token.Token);
                _securityStore.Set(token.Token, storeToken);
            }
        }
    }
}
